//-------------------------------------------------------------------------------------------------------------
// Aginova specific functions
//-------------------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------------------
//   ,ad8888ba,                                                                      88
//  d8"'    `"8b                                                                     ""
// d8'
// 88             ,adPPYba,  8b,dPPYba,  8b       d8  ,adPPYba, 8b,dPPYba, ,adPPYba, 88  ,adPPYba,  8b,dPPYba,
// 88            a8"     "8a 88P'   `"8a `8b     d8' a8P_____88 88P'   "Y8 I8[    "" 88 a8"     "8a 88P'   `"8a
// Y8,           8b       d8 88       88  `8b   d8'  8PP""""""" 88          `"Y8ba,  88 8b       d8 88       88
//  Y8a.    .a8P "8a,   ,a8" 88       88   `8b,d8'   "8b,   ,aa 88         aa    ]8I 88 "8a,   ,a8" 88       88
//   `"Y8888Y"'   `"YbbdP"'  88       88     "8"      `"Ybbd8"' 88         `"YbbdP"' 88  `"YbbdP"'  88       88
//-------------------------------------------------------------------------------------------------------------


//-------------------------------------------------
// UDP Data to formatted Packet
//-------------------------------------------------
// The UDP data should have already been converted to a POJO
function udp2packet(data) {

  // As air temperature can come from either the single temp probe or combined temp and humidity probe (each of
  // which has a different fieldname) we'll need to check if either is present. As these are all strings there
  // doesn't need to be any isNumeric() checks
  var tempVal = data.temp1 || data.tempSHT;
  
  var packet = {
    obs: {
      at:  convertRawTemp(tempVal),
      rst: convertRawTemp(data.irTemp),
      rh:  convertRawHumidity(data.humSHT),
      ms:  convertRawMoisture(data.wetness),
    },
    arrivaltime: new Date(),
    nova_id    : data.SensorID,
    battery    : convertRawBattery(data.battery),
    macaddress : data.Mac,
    probe_id   : data.sensorType,
    charging   : data.ChargingStatus,
  } ;

  return packet;

}
// Export it
exports.udp2packet = udp2packet;



//-------------------------------------------------
// Sentinel temp to celcius
//-------------------------------------------------
function convertRawTemp(raw) {

  if (typeof raw === "undefined") {
    return undefined;
  }

  // Make sure raw is a number
  raw = parseFloat(raw);

  // Convert it
  var converted = (raw - 25000)/ 100; 

  // Round it to ? d.p
  converted = parseFloat(converted.toFixed(2));

  return converted;

}
// Export it
exports.convertRawTemp = convertRawTemp;


//-------------------------------------------------
// Sentinel relative humidity to percent
//-------------------------------------------------
function convertRawHumidity(raw) {

  if (typeof raw === "undefined") {
    return undefined;
  }

  // Make sure raw is a number
  raw = parseFloat(raw);

  // Convert it
  var converted = raw / 100; 

  return converted;

}
// Export it
exports.convertRawHumidity = convertRawHumidity;


//-------------------------------------------------
// Sentinel Moisture to percent
//-------------------------------------------------
function convertRawMoisture(raw) {

  if (typeof raw === "undefined") {
    return undefined;
  }

  // Make sure raw is a number
  raw = parseFloat(raw);

  // Convert it
  var converted = raw / 100; 

  return converted;

}
// Export it
exports.convertRawMoisture = convertRawMoisture;


//-------------------------------------------------
// Convert Battery to Volts
//-------------------------------------------------
function convertRawBattery(raw) {

  // Make sure raw is a number
  raw = parseFloat(raw);

  // Convert it (by dividing by 1000)
  var converted = raw / 1000; 

  // I won't bother round to any dp simply because the raw value only had 4 significant figures anyway and we're
  // simply dividing by 1000.

  return converted;

}
// Export it
exports.convertRawBattery = convertRawBattery;






//-------------------------------------------------------------------------------------------------------------
// 88888888ba               ad88
// 88      "8b             d8"
// 88      ,8P             88
// 88aaaaaa8P' ,adPPYba, MM88MMM ,adPPYba, 8b,dPPYba,  ,adPPYba, 8b,dPPYba,   ,adPPYba,  ,adPPYba,
// 88""""88'  a8P_____88   88   a8P_____88 88P'   "Y8 a8P_____88 88P'   `"8a a8"     "" a8P_____88
// 88    `8b  8PP"""""""   88   8PP""""""" 88         8PP""""""" 88       88 8b         8PP"""""""
// 88     `8b "8b,   ,aa   88   "8b,   ,aa 88         "8b,   ,aa 88       88 "8a,   ,aa "8b,   ,aa
// 88      `8b `"Ybbd8"'   88    `"Ybbd8"' 88          `"Ybbd8"' 88       88  `"Ybbd8"'  `"Ybbd8"'
//-------------------------------------------------------------------------------------------------------------

//-------------------------------------------------
// Important Sentinel Fields
//-------------------------------------------------

// timestamp - number of seconds (not milliseconds) since the sentinel was last reset.

// timedrifted - appears to be a measure of the number of seconds since midnight 1st Jan 2006.


//-------------------------------------------------
// Probe IDs
//-------------------------------------------------

// 312 - Temperature Sensor

// 314 - Combined Temperature And RH sensor (sensorValue1 is temp, sensorValue2 is RH)

// 332 - Moisture Sensor

// 346 - IR Sensor

//-------------------------------------------------
// Example req.body
//-------------------------------------------------

// { sensorID: '100164',
//   sensorKey: '0',
//   timestamp: '572281',
//   battery: '3967',
//   RSSI: '-59',
//   codeVersion: '7.1.22',
//   probeID: '312',
//   prodNum: '254',
//   mac: '74:6a:89:00:13:05',
//   otapStatus: '0',
//   chargingStatus: '0',
//   timedrifted: '280836710',
//   onOffStatus: '1',
//   alarmMode: '0',
//   sampling: '30',  <- i.e. samples every 30 seconds
//   ulp: '1',        <- i.e. uploads for every n samples, so a value of 1 mean sampling rate and upload rate are the same
//   safMode: '0',   <- 1 means store and forward is on, 0 for off.
//   channel: '1',
//   ssid1: 'SKY71892',
//   dhcp: '1',
//   destinationAddr: 'http://178.62.97.120:5454/httpdataatatadata',
//   tempUnit: '0',
//   temp1Drift: '0',
//   temp2Drift: '0',
//   tempshtDrift: '0',
//   humshtDrift: '0',
//   temprtdDrift: '0',
//   temp1UpperAlarm: '0',
//   temp1LowerAlarm: '0',
//   temp2UpperAlarm: '0',
//   temp2LowerAlarm: '0',
//   tempShtUpperAlarm: '0',
//   tempShtLowerAlarm: '0',
//   humShtUpperAlarm: '0',
//   humShtLowerAlarm: '0',
//   tempRtdUpperAlarm: '0',
//   tempRtdLowerAlarm: '0',
//   wetnessUpperAlarm: '0',
//   wetnessLowerAlarm: '0',
//   ApMac: 'c0:3e:0f:2d:a4:6d',
//   inFlash: '0',
//   sensorValue1: '26563' }

//-------------------------------------------------
// Example get store and forward post
//-------------------------------------------------
// { sensorID: '100164',
//   timestamp: '661136',
//   probeID: '312',
//   SFdata: 'c58ebe10b807ffffe38ebe10b807ffff008fbe10b807ffff1d8fbe10b507ffff3c8fbe10b707ffff588fbe10b607ffff778fbe10b707ffff928fbe10b407ffffb08fbe10b407ffffcd8fbe10b207ffff0695be10e968ffff2495be10e968ffff4195be10ea68ffff5f95be10ea68ffff7d95be10eb68ffff',
//   SFnumber: '15' }




