(function() {
  'use strict';

  //-------------------------------------------------
  // Average Controller
  //-------------------------------------------------
  angular
    .module('directApp')
    .controller('AverageController',
      ['$scope', 'udpService', AverageController]);  

  // Keep dependencies in sync!
  function AverageController($scope, udpService) {

    // Bindable Members Up Top
    var vm = this;
    vm.packet          = {};
    vm.averaging       = false;
    vm.startAveraging  = startAveraging;
    vm.stopAveraging   = stopAveraging;
    vm.arr2Avg         = [];
    vm.n2Avg           = 20; // How many obs to average before stopping  
    vm.isNumeric       = isNumeric;
    vm.varOptions      = [];  
    vm.var2Avg         = undefined;
    vm.probeID         = undefined;
    

    activate();
    //------------------

    function activate(){

      resetStats();

    }


    //-------------------------------------------------
    // On New Packet
    //-------------------------------------------------
    $scope.$on('packet:new', function(event, packet) {

      // Update the controllers latest packet object
      vm.packet = packet;

      // Send the packet onto the function that processes it to calculate the average
      processPacket(packet);      

      // If the probe has changed.
      if (packet.probe_id !== vm.probeID) {

        // Stop averaging
        stopAveraging();

        // Use the packet to set the choice of variable to average
        vm.varOptions = whatVarOptions(packet);
        vm.var2Avg = vm.varOptions[0]; // Set as the first by default
        vm.probeID = packet.probe_id;
      } 

      // Update two way binding
      $scope.$apply(function () { });

    });


    //-------------------------------------------------
    // What variables are available
    //-------------------------------------------------
    // Use the packet to determine what variables the user can choose from to average.
    // For the select box
    function whatVarOptions(packet) {

      var varOptions = [];

      if (isNumber(packet.obs.rst)) {
        varOptions.push({
          title: 'Infrared Temperature',
          id: 'rst'
        });
      }

      if (isNumber(packet.obs.at)) {
        varOptions.push({
          title: 'Air Temperature',
          id: 'at'
        });
      }

      if (isNumber(packet.obs.rh)) {
        varOptions.push({
          title: 'Relative Humidity',
          id: 'rh'
        });
      }
      if (isNumber(packet.obs.ms)) {
        varOptions.push({
          title: 'Moisture',
          id: 'ms'
        });
      }

      return varOptions;

    }


    //-------------------------------------------------
    // Start Averaging
    //-------------------------------------------------
    function startAveraging() {

      // Reset everything
      vm.arr2Avg = [];
      resetStats();

      vm.averaging = true;
    }

    //-------------------------------------------------
    // Stop Averaging
    //-------------------------------------------------
    function stopAveraging() {
      vm.averaging = false;
    }

    //-------------------------------------------------
    // Process new packet
    //-------------------------------------------------
    function processPacket(packet) {

      // Are we currenting averaging?
      if (vm.averaging) {

        // Have we got enough observations yet
        if (vm.arr2Avg.length < vm.n2Avg) {

          // Is the new observation valid? This will catch if the probe is removed half way through.
          if (isNumber(packet.obs[vm.var2Avg.id])) {

            // Add the new obs to our array
            vm.arr2Avg.push(packet.obs[vm.var2Avg.id]);

            // Update our stats
            updateStats();

            // If this is the first obs then note when this first obs was
            if (vm.arr2Avg.length === 1) {
              vm.firstAvgTime = new Date(packet.arrivaltime);
            }
            // We'll also note down the time of this latest obs so we can calculate the time between the first and
            // last obs.
            vm.latestAvgTime = new Date(packet.arrivaltime);

            if (vm.arr2Avg.length === vm.n2Avg) {

              // We've got all the values we needs, so can stop averaging now.
              vm.averaging = false;
              
            }
      
          }

        }
        
      }

    }


    //-------------------------------------------------
    // Update Statistics
    //-------------------------------------------------
    function updateStats() {

      vm.mean     = calcMean(vm.arr2Avg);
      vm.variance = calcVariance(vm.arr2Avg);
      vm.stddev   = calcStdDev(vm.arr2Avg);
      vm.min      = calcMin(vm.arr2Avg);
      vm.max      = calcMax(vm.arr2Avg);

    }


    //-------------------------------------------------
    // Calculate sum
    //-------------------------------------------------
    function calcSum(array) {

      var sum = 0;
      for (var i = 0; i < array.length; i++) {
        sum += array[i];
      }
      return sum;

    }


    //-------------------------------------------------
    // Calculate mean average
    //-------------------------------------------------
    function calcMean(array) {

      return calcSum(array) / array.length;

    }


    //-------------------------------------------------
    // Calculate Variance
    //-------------------------------------------------
    function calcVariance(array) {
      var mean = calcMean(array);
      var sumOfSquaredDiffs = calcSum(array.map(function(num) {
        return Math.pow(num - mean, 2);
      }));
      // You have a choice here of dividing by n or n-1 depending on whether you want a biased or unbiased
      // estimator. I've gone with n-1 as this is what matlab uses by default.
      return sumOfSquaredDiffs / (array.length - 1);
    } 


    //-------------------------------------------------
    // Calculate Standard Deviation
    //-------------------------------------------------
    function calcStdDev(array) {

      return Math.sqrt(calcVariance(array));

    }



    //-------------------------------------------------
    // Min & Max
    //-------------------------------------------------
    function calcMin(array) {
      return Math.min.apply(Math, array);
    }

    function calcMax(array) {
      return Math.max.apply(Math, array); 
    }


    //-------------------------------------------------
    // Reset Stats
    //-------------------------------------------------
    function resetStats() {
      vm.mean            = undefined;
      vm.variance        = undefined;
      vm.min             = undefined;
      vm.max             = undefined;
      vm.firstAvgTime    = undefined;
      vm.latestAvgTime   = undefined;      
    }


    //-------------------------------------------------
    // Is Numeric?
    //-------------------------------------------------
    // Here's a handy function to check if a variable is numeric or not (numeric strings will pass test).
    function isNumeric(n){
      return !isNaN(parseFloat(n)) && isFinite(n);
    }

    //-------------------------------------------------
    // Is Number
    //-------------------------------------------------
    function isNumber(n) {
      return n === parseFloat(n);
    }



  }//<- Controller

})();//<- IIFE
