(function() {
  'use strict';

  //-------------------------------------------------
  // Config Controller
  //-------------------------------------------------
  angular
    .module('directApp')
    .controller('ConfigController',
      ['$timeout', '$scope', 'udpService', 'storedb', ConfigController]);

  // Keep dependencies in sync!
  function ConfigController($timeout, $scope, udpService, storedb) {


    // Bindable Members Up Top
    var vm = this;
    vm.configure        = configure;
    vm.fd               = {};
    vm.securityModes    = makeSecurityModes();
    vm.wepKeyIds        = makeWepKeyIds();
    vm.dhcpOptions      = makeDchpOptions();
    vm.storeSettings    = storeSettings;
    vm.removeSetting    = removeSetting;
    vm.useSetting       = useSetting;
    vm.showPrevious     = false;
    vm.smartName        = smartName;
    vm.storedOK         = false;
    vm.packet           = {};
    vm.isNumeric        = isNumeric;
    vm.status           = 'primed';
    
    activate();
    //------------------

    function activate(){

      basicFormInit();

      // Find previously saved remote config settings
      vm.storedSettings = retrieveSettings();
      // console.log(vm.storedSettings);

    }


    //-------------------------------------------------
    // Formdata initialisations
    //-------------------------------------------------
    function basicFormInit() {
      vm.fd = {
        name: 'My Settings',
        security: vm.securityModes[1].name,
        wiboxAddress: 'https://wifi.wintersense.com/packets/v1',
        samplingRate: 300,
        uploadRate: 6,
        saf: true,
        wepKeyId: vm.wepKeyIds[0],
      };

      setDefaultDHCP();
    }

    function setDefaultDHCP() {
      vm.fd.dhcp = vm.dhcpOptions[0].value;
    }

    function makeSecurityModes() {

      return [      
        {
          name: 'Open'
        },        
        {
          name: 'WPA Personal'
        },
        {
          name: 'WPA Enterprise'
        },
        {
          name: 'WEP'
        }        
      ];

    }


    //-------------------------------------------------
    // Make WEP Key Ids
    //-------------------------------------------------
    function makeWepKeyIds() {
      return ['0','1','2','3'];
    }


    function makeDchpOptions() {

      return [
        {
          title: 'Don\'t change',
          value: ''
        },
        {
          title: 'On',
          value: '1'
        },
        {
          title: 'Off',
          value: '0'
        }
      ];

    }


    //-------------------------------------------------
    // New Packet
    //-------------------------------------------------
    $scope.$on('packet:new', function(event, data) {

      vm.packet = data;

      // Unlike $http or $timeout the udp server is a third party package triggering events, as such it won't
      // update the two binding like angular's own services will, this we need this $apply.
      $scope.$apply(function () { });      

    });

    //-------------------------------------------------
    // Is Numeric?
    //-------------------------------------------------
    // Here's a handy function to check if a number is numeric or not
    function isNumeric(n){
      return !isNaN(parseFloat(n)) && isFinite(n);
    }


    //-------------------------------------------------
    // Configure
    //-------------------------------------------------
    function configure(fd) {

      // Tidy the formdata up a bit first
      var tidied = tidyFormdata(fd);

      vm.status = 'sending';

      // Tell the udpService to send the new settings to the sensor (after checking them first)
      udpService.sendConfig(tidied, function(err, sentSettings) {

        if (err) {
          vm.status = 'failed';
          alert(err.message);
        } else {
          vm.status = 'sent';
        }

        // Need an apply here for the status change to be seen in the view
        $scope.$apply(function () { });     

        // Reset the button after a delay
        $timeout(function() {
          vm.status = 'primed';
        }, 2000);        
  

      });

    }


    //-------------------------------------------------
    // Tidy formdata
    //-------------------------------------------------
    function tidyFormdata(fd) {

      // Formdata formatting
      // If vm.fd.dhcp is '', i.e. don't change then we can pull it from our form data as the server will expect
      // either a value of 1 or 0.
      if (fd.dhcp === '') {
        delete fd.dhcp;
      }
      // Remove any fields where the user has cleared the box, this is primarily for optional fields where an
      // empty string or null value would upset our server now that the field is no longer undefined.
      if (fd.samplingRate === null) {
        delete fd.samplingRate;
      }
      if (fd.uploadRate === null) {
        delete fd.uploadRate;
      }
      if (fd.dns1 === '') {
        delete fd.dns1;
      }
      if (fd.dns2 === '') {
        delete fd.dns2;
      }

      return fd; 

    }


    //-------------------------------------------------
    // Store current settings to databases
    //-------------------------------------------------
    function storeSettings(fd) {

      // Feels like a good idea to clone this
      var clone = {}; // Important to initialise this first
      angular.copy(fd, clone);

      // // Tidy it up
      var tidied = tidyFormdata(clone);

      storedb('settings').insert(tidied, function(err, result) {
        if(err) {
          console.log(err);
        } else {

          // Now update the list of settings
          vm.storedSettings = retrieveSettings();

          vm.storedOK = true;

          $timeout(function() {
            vm.storedOK = false;
          }, 2000);

        }

      });

    }


    //-------------------------------------------------
    // Retrieve all stored settings
    //-------------------------------------------------
    function retrieveSettings() {

      return storedb('settings').find() || [];

    }

    //-------------------------------------------------
    // Use stored setting as formdata
    //-------------------------------------------------
    function useSetting(settingObject) {

      vm.fd = settingObject;

      // If dhcp is undefined then set it as one of the options so the select box looks good
      if (typeof vm.fd.dhcp === 'undefined') {
        setDefaultDHCP();
      }

      // Let's hide the previous settings again now the user has made their choice
      vm.showPrevious = false;

    }

    //-------------------------------------------------
    // Remove setting from database
    //-------------------------------------------------
    function removeSetting(settingObject) {

      storedb('settings').remove({_id: settingObject._id}, function(err){
        if (err) {
          console.log(err);
        } else {
          vm.storedSettings = retrieveSettings();
        }
      });      

    }


    //-------------------------------------------------
    // Name the settings based on formdata
    //-------------------------------------------------
    function smartName(fd) {

      if (fd.ssid) {
        fd.name = fd.ssid + ' settings';
      } else {
        fd.name = 'My Settings';
      }

    }



  }//<- Controller

})();//<- IIFE





