(function() {
  'use strict';

  //-------------------------------------------------
  // Package Dependencies
  //-------------------------------------------------
  var dgram       = require('dgram'); // More info here: https://nodejs.org/api/dgram.html
  var udpServer   = dgram.createSocket('udp4');
  var aginova     = require('./assets/js/aginova.js');
  var querystring = require('querystring');
  var check       = require('check-types'); 
  
  //-------------------------------------------------
  // Create Service
  //-------------------------------------------------
  angular
    .module('directApp')
    .service('udpService',
      ['$rootScope', '$timeout', udpService]);
    // Better as a factory?

  // Keep dependencies in sync!
  function udpService($rootScope, $timeout) {

    // Accessible Members Up Top
    /* jshint validthis: true */
    var vm = this;
    vm.packet = {};
    vm.sendConfig = sendConfig;


    //-------------------------------------------------
    // Setup UDP Server
    //-------------------------------------------------

    // Set what port and host the UDP server should watch
    var port = 54521;
    var host = '193.168.101.255'; //<-- Sentinel should always broadcast to this address according to Karl

    // Listening event is emitted when a socket starts listening for datagrams. This happens as soon as UDP sockets
    // are created.
    udpServer.on('listening', function () {
      var address = udpServer.address();
      console.log('UDP Server listening on ' + address.address + ":" + address.port);
    });

    // Error event is emitted when an error occurs
    // Could watch for this event in the controller instead?
    udpServer.on('error', function (err) {

      // console.log(err);
      // console.log('UDP Error Event!!!!! ');
      // console.log("Are you connected to the sentinel?");

      // Wait a few seconds then start the UDP server again, hopefully the sentinel will now be connected.
      $timeout(function() {

        udpServer.bind(port, host);
      
      }, 2000);      


    });


    //-------------------------------------------------
    // Sentinel Settings
    //-------------------------------------------------
    // Set up a few defaults for the sentinel
    // This is the address/port on which the sentinel will be waiting for configuration packets
    var sentinelAddress = '193.168.101.1'; // <-- Not sure yet if this will always be fixed?
    var sentinelPort    = 50001; // <-- This should be fixed


    //-------------------------------------------------
    // Message Event
    //-------------------------------------------------
    // A message event is emitted when a new datagram is available on a socket.
    // message - Buffer object. The message.
    // remote  - Object. Remote address information.
    udpServer.on('message', function (message, remote) {
      
      // console.log('\n' +  'Message from ' + remote.address + ':' + remote.port);
      // console.log(Buffer.isBuffer(message));
      // console.log(Buffer.isEncoding('utf8'));

      if (remote.address !== sentinelAddress) {
        throw new Error("It looks as though the sentinels address can change, thus may need to change code");
        // If this gets thrown then you'll want to change the code so the sentinelAddress gets updated.
        // Basically I don't know yet whether the sentinel will always be on ip address 193.168.101.1
      }

      // Convert the message (which is a Buffer) to a string first
      var dataString = message.toString();
      // And then from a query string to a plain old javascript object
      var data = querystring.parse(dataString);

      var packet = aginova.udp2packet(data);

      $rootScope.$broadcast('packet:new', packet);

    });



    //-------------------------------------------------
    // Send Configuation Packet
    //-------------------------------------------------
    // settingsIn is the settings defined in the user completed form
    function sendConfig(settingsIn, callback) {

      console.log(settingsIn);

      //---------
      // Checks
      //---------
      // Wibox Address
      if (check.not.unemptyString(settingsIn.wiboxAddress)) {
        return callback(new Error('You must provide a server address'));
      } 
      if (settingsIn.wiboxAddress.slice(0,7) !== 'http://' && settingsIn.wiboxAddress.slice(0,8) !== 'https://') {
        return callback(new Error('Server Address must begin with http:// or https://'));
      }
      // Decided that having /httpdata at the end doesn't need to be compulsory in case the user is using some
      // non-standard implentation
      // if (settingsIn.wiboxAddress.slice(settingsIn.wiboxAddress.length-9, settingsIn.wiboxAddress.length) !== '/httpdata') {
      //   return callback(new Error('Server address must end with /httpdata'));
      // }   
      // Security Type
      if (check.not.unemptyString(settingsIn.security)) {
        return callback(new Error('No security type has been specified'));
      }         
      // SSID
      if (check.not.unemptyString(settingsIn.ssid)) {
        return callback(new Error('You must provide a SSID'));
      }     
      // Password (if network is not open)
      if (settingsIn.security !== 'Open' && check.not.unemptyString(settingsIn.password)) {
        return callback(new Error('The Wi-Fi Network\'s password is required when using ' + settingsIn.security));
      } 
      // Username (if using Enterprise)
      if (settingsIn.security === 'WPA Enterprise' && check.not.unemptyString(settingsIn.username)) {
        return callback(new Error('You must provide the username of the WiFi Network'));
      } 
      // Sampling Rate
      if (typeof settingsIn.samplingRate !== 'undefined') {     
        if (check.not.integer(settingsIn.samplingRate) || settingsIn.samplingRate < 1) {
          return callback(new Error('The Sampling Rate should be a whole number larger than 0'));
        }
      }
      // Upload Rate  
      if (typeof settingsIn.uploadRate !== 'undefined') { 
        if (check.not.integer(settingsIn.uploadRate) || settingsIn.uploadRate < 1) {
          return callback(new Error('The Upload Rate should be a whole number larger than 0'));
        }
      }
      // Store and forward
      if (check.not.boolean(settingsIn.saf)) {
        return callback(new Error('Store and Forward should be a boolean'));
      } 
      // DHCP
      if (typeof settingsIn.dhcp !== 'undefined' && settingsIn.dhcp !== '1' && settingsIn.dhcp !== '0') {
        return callback(new Error("DHCP should either be undefined, '1', or '0'"));
      }   

      // Convert the settingsIn to an object with key's that match those that the sentinel recognises
      var settingsOut = {};
      // Begin with the common settings
      settingsOut.CommMode              = '2'; // switches sentinel to remote mode
      settingsOut.Changewiboxaddress    = settingsIn.wiboxAddress;
      settingsOut.ChangeSSID            = settingsIn.ssid;
      settingsOut.ChangeStoreAndForward = settingsIn.saf ? 1 : 0;

      // Now the settings that depend on security type
      if (settingsIn.security === 'WPA Personal') {

        settingsOut.ChangeWpa = settingsIn.password;

      } else if (settingsIn.security === 'WPA Enterprise') {

        if (check.not.unemptyString(settingsIn.username)) {
          return callback(new Error('You must provide a username when using WPA Enterprise'));
        } 
        settingsOut.ChangeEapUser     = settingsIn.username;
        settingsOut.ChangeEapPassword = settingsIn.password;
        settingsOut.ChangeSecurity    = '4';

      } else if (settingsIn.security === 'WEP') {  

        if (check.not.unemptyString(settingsIn.wepKeyId)) {
          return callback(new Error('You must provide a Key ID when using WEP'));
        } 

        settingsOut.ChangeWepKey   = settingsIn.password;    
        settingsOut.Changewepkeyid = settingsIn.wepKeyId;    
        settingsOut.ChangeSecurity = '2';

      } else if (settingsIn.security === 'Open') {  
        // If the security type is 'Open' then we don't need to add anything.
      } else {
        return callback(new Error('Unknown Security type'));
      }

      // Set these as being optional in case the user wants to keep using whatever the settings were before.
      if (typeof settingsIn.samplingRate !== "undefined") { 
        settingsOut.ChangeSampling = settingsIn.samplingRate;
      }
      if (typeof settingsIn.uploadRate !== "undefined") { 
        settingsOut.ChangeULP = settingsIn.uploadRate;
      }

      if (settingsIn.dhcp === '1') {
        settingsOut.Changedhcp = '1';
      }

      if (settingsIn.dhcp === '0') {
        settingsOut.Changedhcp = '0';
        // Check and set the additional settings required when DHCP is not in use
        if (check.not.unemptyString(settingsIn.ipaddress)){
          return callback(new Error('IP address is required when DHCP is disabled'));
        }
        if (check.not.unemptyString(settingsIn.subnet)){
          return callback(new Error('Subnet mask is required when DHCP is disabled'));
        }
        if (check.not.unemptyString(settingsIn.gateway)){
          return callback(new Error('Default gateway is required when DHCP is disabled'));
        }
        settingsOut.Changeipaddress = settingsIn.ipaddress;
        settingsOut.Changesubnet    = settingsIn.subnet;
        settingsOut.Changegateway   = settingsIn.gateway;    
      }   

      if (typeof settingsIn.dns1 !== "undefined") {
        if (check.not.unemptyString(settingsIn.dns1)) {
          return callback(new Error('The Primary DNS Server should be an unempty string'));
        }
      }

      if (typeof settingsIn.dns2 !== "undefined") {
        if (check.not.unemptyString(settingsIn.dns2)) {
          return callback(new Error('The Secondary DNS Server should be an unempty string'));
        }     
      }   

      // For testing
      // var msgString = 'ChangeSSID=androidhotspot&ChangeWpa=brrChilly&Changewiboxaddress=http://wintersense.com:8080/httpdata&CommMode=2&';
      
      // Convert the settingsOut to a query string
      var msgString = querystring.stringify(settingsOut) + '&'; // Adding +'&' does prove to be vital sometimes.
      // Undo the auto-escaping
      msgString = querystring.unescape(msgString);
      // console.log(msgString);
      var message = new Buffer(msgString);


      // Send it
      var udpSender = dgram.createSocket('udp4');
      udpSender.send(message, 0, message.length, sentinelPort, sentinelAddress, function(err, bytes) {

        if (err) return callback(err);
        console.log('UDP configution packet sent to ' + sentinelAddress +':'+ sentinelPort);
        udpSender.close(); //Don't think I need this?
        // Might as well return the final settingsOut object in case I want to display it on the view
        callback(null, settingsOut);

      });


    } //<-sendConfig




    //-------------------------------------------------
    // Start Listening
    //-------------------------------------------------
    // It's important to call .bind using this asynchronous approach now.
    udpServer.bind(port, function() {

    });


  }//<- Service

})();//<- IIFE














